@@ -1,5 +1,5 @@
 /* -*- c -*-
-    $Id: cdio.h,v 1.35 2004/02/21 13:10:51 rocky Exp $
+    $Id: cdio.h,v 1.37 2004/03/20 22:46:57 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -146,6 +146,8 @@ extern "C" {
     To find a CD-drive of any type, use the mask CDIO_FS_MATCH_ALL.
   
     NULL is returned if we couldn't get a default device.
+    It is also possible to return a non NULL but after dereferencing the 
+    the value is NULL. This also means nothing was found.
   */
   char ** cdio_get_devices_with_cap (char* search_devices[],
 				     cdio_fs_anal_t capabilities, bool any);
@@ -278,21 +280,21 @@ extern "C" {
    from lsn. Returns 0 if no error. 
   */
   int cdio_read_mode1_sector (const CdIo *obj, void *buf, lsn_t lsn, 
-			      bool is_form2);
+			      bool b_form2);
   
   /*!
     Reads nblocks of mode1 sectors from cd device into data starting
     from lsn. Returns 0 if no error. 
   */
   int cdio_read_mode1_sectors (const CdIo *obj, void *buf, lsn_t lsn, 
-			       bool is_form2, unsigned int num_sectors);
+			       bool b_form2, unsigned int num_sectors);
   
   /*!
     Reads a single mode2 sector from cd device into data starting
     from lsn. Returns 0 if no error. 
   */
   int cdio_read_mode2_sector (const CdIo *obj, void *buf, lsn_t lsn, 
-			      bool is_form2);
+			      bool b_form2);
   
   /*!
     Reads nblocks of mode2 sectors from cd device into data starting
@@ -300,7 +302,7 @@ extern "C" {
     Returns 0 if no error. 
   */
   int cdio_read_mode2_sectors (const CdIo *obj, void *buf, lsn_t lsn, 
-			       bool is_form2, unsigned int num_sectors);
+			       bool b_form2, unsigned int num_sectors);
   
   /*!
     Set the arg "key" with "value" in the source device.
