@@ -1,7 +1,7 @@
 #!/bin/sh
 #$Id: check_cue.sh.in,v 1.15 2003/10/06 04:04:05 rocky Exp $
 # Tests to see that BIN/CUE file iamge reading is correct (via cd-info). 
-if test -n "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+if test -n "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
   vcd_opt='--no-vcd'
 fi
 
@@ -41,7 +41,7 @@ else
 fi
 
 fname=vcd_demo
-if test -z "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+if test -z "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
   right=${srcdir}/${fname}.right
 else
   right=${srcdir}/${fname}_vcdinfo.right
