@@ -1,7 +1,7 @@
 #!/bin/sh
 #$Id: check_nrg.sh.in,v 1.9 2003/10/06 04:04:05 rocky Exp $
 
-if test -n "-L/usr/local/lib -lvcdinfo -lvcd -liso9660 -lcdio  " ; then
+if test -n "-L/usr/local/lib -lvcdinfo -liso9660 -lcdio -lm  " ; then
   vcd_opt='--no-vcd'
 fi
 
